@@ -30,11 +30,11 @@
 // Extractor is also used outside of here, but only trivially.
 //
 // Nothing in here is dependent on the DSL surface syntax.  It does
-// assume logit(), announce(), croak(), throw() and catch() do sane things.
+// assume logit(), croak(), throw() and catch() do sane things.
 // The code also assumes a control block that includes a baton for
 // reports.
 //
-// Copyright by Eric S. Raymond
+// SPDX-FileCopyrightText: Copyright Eric S. Raymond <esr@thyrsus.com>
 // SPDX-License-Identifier: BSD-2-Clause
 
 package main
@@ -429,7 +429,8 @@ func (ge *GitExtractor) gatherAllReferences(rs *RepoStreamer) error {
 				return fmt.Errorf("warning: atttribution in tag %s garbled: %v", tag, err)
 			}
 			// committish isn't a mark; we'll fix that later
-			tagobj := *newTag(nil, tag, objecthash, attrib, comment)
+			tagobj := *newTag(nil, tag, objecthash, comment)
+			tagobj.tagger = *attrib
 			rs.tags = append(rs.tags, tagobj)
 		}
 		rs.baton.twirl()
@@ -675,7 +676,7 @@ func (he *HgExtractor) byLine(rs *RepoStreamer, cmd []string, errfmt string,
 	return nil
 }
 
-//gatherRevisionIDs gets the topologically-ordered list of revisions and parents.
+// gatherRevisionIDs gets the topologically-ordered list of revisions and parents.
 func (he *HgExtractor) gatherRevisionIDs(rs *RepoStreamer) error {
 	// Belated initialization
 	he.base = rs
