@@ -13,6 +13,8 @@ import (
 	"strconv"
 	"strings"
 	"testing"
+
+	shlex "github.com/anmitsu/go-shlex"
 )
 
 func TestMain(m *testing.M) {
@@ -68,6 +70,13 @@ func TestRegexp(t *testing.T) {
 		{"aaa(/|$)", "aaa/", true},
 		{"aaa(/|$)", "aaab/", false},
 		{"aaa(/|$)", "aaab", false},
+		// Following cases check the ability to derect
+		// reference cookies.  Could be screwed up if the
+		// regexp engine makes \ and [ interact in
+		// unexpected ways.
+		{`\[\[CVS:[^:\]]+:[0-9.]+\]\]`, "xxx [[CVS:foo:1.1]] yyyy ", true},
+		{`\[\[SVN:[0-9]+\]\]`, "xxx [[SVN:23]] yyyy", true},
+		{`\[\[SVN:[0-9]+\]\]`, "xxx [[SVN:abba]] yyyy", false},
 	}
 	for _, item := range tests {
 		re := regexp.MustCompile(item.pattern)
@@ -75,6 +84,179 @@ func TestRegexp(t *testing.T) {
 	}
 }
 
+func TestIgnoreCompatibility(t *testing.T) {
+	type testEntry struct {
+		line  string
+		vcs   string
+		match bool
+	}
+	tests := []testEntry{
+		// CVS
+		{"#Comment", "cvs", true}, // because ignEXPORT
+		{"foobar", "cvs", true},
+		{"*.a", "cvs", true},
+		{"!*.a", "cvs", false},
+		{".py[co]", "cvs", true},
+		{".netrc~", "cvs", true},
+		{"x[a-z]y", "cvs", true},
+		{"x[!0-9]y", "cvs", true},
+		// SVN
+		{"#Comment", "svn", true}, // because ignEXPORT
+		{"foobar", "svn", true},
+		{"*.a", "svn", true},
+		{"!*.a", "svn", true},
+		{".py[co]", "svn", true},
+		{".netrc~", "svn", true},
+		{"x[a-z]y", "svn", true},
+		{"x[!0-9]y", "svn", true},
+		// bzr/brz
+		{"#Comment", "bzr", true},
+		{"foobar", "bzr", true},
+		{"*.a", "bzr", true},
+		{"!*.a", "bzr", true},
+		{".py[co]", "bzr", true},
+		{".netrc~", "bzr", true},
+		{"x[a-z]y", "bzr", true},
+		{"x[!0-9]y", "bzr", true},
+		// git
+		{"#Comment", "git", true},
+		{"foobar", "git", true},
+		{"*.a", "git", true},
+		{"!*.a", "git", true},
+		{".py[co]", "git", true},
+		{".netrc~", "git", true},
+		{"x[a-z]y", "git", true},
+		{"x[!0-9]y", "git", true},
+		// src
+		{"#Comment", "src", true},
+		{"foobar", "src", true},
+		{"*.a", "src", true},
+		{"!*.a", "src", true},
+		{".py[co]", "src", true},
+		{".netrc~", "src", true},
+		{"x[a-z]y", "src", true},
+		{"x[!0-9]y", "src", false},
+	}
+	var reLatch bool
+	for testnum, item := range tests {
+		cvs := findVCS("cvs")
+		if _, v := translateIgnoreLine(&reLatch, cvs, findVCS(item.vcs), item.line); (v != nil) == item.match {
+			t.Errorf("TestIgnoreCompatibility %d, %s %q: expected %v, saw %v",
+				testnum+1, item.vcs, item.line, item.match, v)
+		}
+	}
+}
+
+func TestIgnoreTranslation(t *testing.T) {
+	type testEntry struct {
+		line       string
+		svcs, tvcs string
+		expect     string
+		reversible bool
+	}
+	tests := []testEntry{
+		// Moving between globs abd regexps
+		{`#Comment`, `git`, `darcs`, `#Comment`, true},
+		{`foobar`, `git`, `darcs`, `foobar$`, true},
+		{`fo?bar`, `git`, `darcs`, `fo.bar$`, true},
+		{`*.obj`, `git`, `darcs`, `.*\.obj$`, true},
+		{`*.py[co]`, `git`, `darcs`, `.*\.py[co]$`, true},
+		{`a[.]b$`, `darcs`, `git`, `a[.]b`, true},
+		{`a[?]b$`, `darcs`, `git`, `a[?]b`, true},
+		{`a[.*]b$`, `darcs`, `git`, `a[.*]b`, true},
+		// The omly glob translation we can do
+		{`[!a-z]`, `git`, `hg`, `[^a-z]`, false},
+		//Check for quirks
+		{`!!foobar`, `bzr`, `git`, `#!!foobar`, false},
+		{`foobar**`, `git`, `hg`, `#foobar**`, false},
+		{`foo\?bar`, `git`, `bzr`, `#foo\?bar`, false},
+		// Check for forced anchoring
+		{`foobar`, `cvs`, `hg`, `./foobar`, false},
+	}
+	var reLatch bool
+	for testnum, item := range tests {
+		svcs := findVCS(item.svcs)
+		tvcs := findVCS(item.tvcs)
+		reLatch = svcs.hasCapability(ignRE)
+		if seen, e := translateIgnoreLine(&reLatch, svcs, tvcs, item.line); seen != item.expect {
+			t.Errorf("TestIgnoreTranslation %d (forward), %s->%s %q: expected %q, saw %q, error %v",
+				testnum+1, item.svcs, item.tvcs, item.line, item.expect, seen, e)
+		}
+		if item.reversible {
+			reLatch = tvcs.hasCapability(ignRE)
+			if seen, e := translateIgnoreLine(&reLatch, tvcs, svcs, item.expect); seen != item.line {
+				t.Errorf("TestIgnoreTranslation %d (backward), %s->%s %q: expected %q, saw %q, error %v",
+					testnum+1, item.tvcs, item.svcs, item.expect, item.line, seen, e)
+			}
+		}
+	}
+}
+
+func TestCookies(t *testing.T) {
+	type testEntry struct {
+		pattern  string
+		text     string
+		expected bool
+	}
+	tests := []testEntry{
+		{dottedNumeric, " 1.2 ", true},
+		{dottedNumeric, "A1.2 ", false},
+		{dottedNumeric, " 1.2.3 ", true},
+		{dottedNumeric, " 1.2.3\n", true},
+		{dottedNumeric, " 1.2.3. ", true},
+		{tokenNumeric, " 1.2 ", false},
+		{tokenNumeric, " 23 ", true},
+		{tokenNumeric, " 23. ", true},
+		{tokenNumeric, " 23.\n", true},
+		{tokenNumeric, " 23A", false},
+		{tokenNumeric, "A23A", false},
+		{longGitHash, "0102030405060708090A0102030405060708090A ", true},
+		{longGitHash, "0102030405060708090A0102030405060708090A.", true},
+		{shortGitHash, "010203 ", true},
+		{shortGitHash, "010203.", true},
+	}
+	for testnum, test := range tests {
+		if regexp.MustCompile(test.pattern).MatchString(test.text) != test.expected {
+			t.Errorf("TestCookies %d, %s %q: expected %v",
+				testnum+1, test.pattern, test.text, test.expected)
+		}
+	}
+}
+
+func TestShlex(t *testing.T) {
+	type testEntry struct {
+		text  string
+		check []string
+	}
+	tests := []testEntry{
+		// Smoke test from the shlex readme
+		{`cp -Rdp "file name" 'file name2' dir\ name`,
+			[]string{"cp", "-Rdp", "file name", "file name2", "dir name"}},
+		// Check that escaping escapes works
+		{`dir\\name`, []string{`dir\name`}},
+		// Check that effect on alphabetics
+		{`dir\xname`, []string{`dirxname`}},
+		// Check effect on our regular-expression backreferences either
+		{`dir${1}name`, []string{`dir${1}name`}},
+	}
+	for testnum, item := range tests {
+		tokens, err := shlex.Split(item.text, true)
+		if err != nil {
+			t.Errorf("TestShlex %d: tokenization failed: %s", testnum, err)
+			continue
+		}
+		if len(item.check) != len(tokens) {
+			t.Errorf("TestShlex %d: expecting %d tokens (%s) but saw %d (%s)", testnum, len(item.check), item.check, len(tokens), tokens)
+			continue
+		}
+		for n, tok := range tokens {
+			if tok != item.check[n] {
+				t.Errorf("TestShlex %d: token %d, expected %s, saw %s", testnum, n+1, item.check[n], tok)
+			}
+		}
+	}
+}
+
 func TestBackreferences(t *testing.T) {
 	from := []byte("aaabxbcccc")
 	pattern := "b(.)b"
@@ -155,7 +337,13 @@ func TestOrderedStringSet(t *testing.T) {
 	if sum[0] != "a" || sum[1] != "b" || sum[2] != "c" || sum[4] != "e" || len(sum) != 5 {
 		t.Errorf("unexpected result of set union: %v", sum)
 	}
+
+	ts10 := ts8.Clone()
+	if !ts10.Equal(ts8) {
+		t.Error("Set cloning failed.")
+	}
 }
+
 func TestStringSet(t *testing.T) {
 	ts := newStringSet("a", "b", "c")
 
@@ -225,6 +413,11 @@ func TestStringSet(t *testing.T) {
 	if !(sum.Contains("a") && sum.Contains("b") && sum.Contains("c") && sum.Contains("e") && sum.Len() == 5) {
 		t.Errorf("unexpected result of set union: %v", sum)
 	}
+
+	ts10 := ts8.Clone()
+	if !ts10.Equal(ts8) {
+		t.Error("Set cloning failed.")
+	}
 }
 
 func SetEqual(s, other selectionSet) bool {
@@ -308,6 +501,11 @@ func TestSelectionSet(t *testing.T) {
 	if sum.Fetch(0) != 1 || sum.Fetch(1) != 2 || sum.Fetch(2) != 3 || sum.Fetch(4) != 5 || sum.Size() != 5 {
 		t.Errorf("unexpected result of set union: %v", sum)
 	}
+
+	ts10 := ts8.Clone()
+	if ts8.String() != ts10.String() {
+		t.Error("Selection set cloning failed.")
+	}
 }
 
 func TestOrderedMap(t *testing.T) {
@@ -773,7 +971,8 @@ func TestTag(t *testing.T) {
 	repo := newRepository("fubar")
 	defer repo.cleanup()
 	attr1, _ := newAttribution("jrh <jrh> 1456976347 -0500")
-	t1 := newTag(repo, "sample1", ":2", attr1, "Sample tag #1\n")
+	t1 := newTag(repo, "sample1", ":2", "Sample tag #1\n")
+	t1.tagger = *attr1
 	repo.events = append(repo.events, t1)
 	if !strings.Contains(t1.Comment, "Sample") {
 		t.Error("expected string not found in tag Comment")
@@ -805,7 +1004,6 @@ Test to be sure we can read in a tag in inbox format.
 		t.Fatalf("On first read: %v", err)
 	}
 	var t2 Tag
-	t2.tagger, _ = newAttribution("")
 	t2.emailIn(msg, false)
 
 	assertEqual(t, "sample2", t2.tagname)
@@ -814,6 +1012,14 @@ Test to be sure we can read in a tag in inbox format.
 	if !t1.decodable() {
 		t.Errorf("%q was expected to be decodable, is not", t1.String())
 	}
+
+	if !t1.clone().tagger.isValid() {
+		t.Errorf("cloning of tag %q failed, tagger invlid", t1.tagname)
+	}
+
+	if t1.String() != t1.clone().String() {
+		t.Errorf("cloning of tag %q failed, stringification mismatch", t1.tagname)
+	}
 }
 
 func TestBranchname(t *testing.T) {
@@ -1092,8 +1298,8 @@ Example commit for unit testing, modified.
 	hackcheck := "------------------------------------------------------------------------\nEvent-Number: 43\nEvent-Mark: :2\nCommitter: J. Random Hacker <jrh@foobar.com>\nCommitter-Date: Wed, 02 Mar 2016 22:39:07 -0500\nAuthor: Tim the Enchanter <esr@thyrsus.com>\nAuthor-Date: Mon, 14 Mar 2016 23:32:27 +0000\nCheck-Text: Example commit for unit testing, modified.\n\nExample commit for unit testing, modified.\n"
 	assertEqual(t, commit.emailOut(nullOrderedStringSet, 42, nil), hackcheck)
 
-	attr1, _ := newAttribution("jrh <jrh> 1456976347 -0500")
-	newTag(repo, "sample1", ":2", attr1, "Sample tag #1\n")
+	//attr1, _ := newAttribution("jrh <jrh> 1456976347 -0500")
+	newTag(repo, "sample1", ":2", "Sample tag #1\n")
 
 	if len(commit.attachments) != 1 {
 		t.Errorf("tag attachment failed: %d", len(commit.attachments))
@@ -1380,10 +1586,34 @@ M 100644 :3 README
 	if !stringSliceEqual(saw2, exp2) {
 		t.Errorf("saw branchset %v, expected %v", saw2, exp2)
 	}
-	saw3 := repo.branchmap()
-	exp3 := map[string]string{"refs/heads/master": ":4"}
+	saw3 := repo.branchtipmap()
+	exp3 := map[string]*Commit{"refs/heads/master": repo.markToEvent(":4").(*Commit)}
 	if !reflect.DeepEqual(saw3, exp3) {
-		t.Errorf("saw branchmap %v, expected %v", saw3, exp3)
+		t.Errorf("saw branchtipmap %v, expected %v", saw3, exp3)
+	}
+	saw4 := repo.branchrootmap()
+	exp4 := map[string]*Commit{"refs/heads/master": repo.markToEvent(":2").(*Commit)}
+	if !reflect.DeepEqual(saw4, exp4) {
+		t.Errorf("saw branchrootmap %v, expected %v", saw4, exp4)
+	}
+
+	// Minpr tests that we put here because they need a scratch repostory
+	rs := newReset(repo, "refs/heads/foobar", ":4", "")
+	//rs.committish = ":4"
+	//rs.ref = "refs/heads/foobar"
+	if rs.String() != rs.clone().String() {
+		t.Errorf("reset cloning failed")
+	}
+	c := newCallout("<2023-09-17T14:54:20Z>")
+	c.branch = "refs/heads/foobar"
+	//c.mark = "<2023-09-17T14:54:20Z>"
+	if c.String() != c.clone().String() {
+		t.Errorf("callout cloning failed")
+	}
+	p := newPassthrough(repo, "foozle")
+	p.text = "foozle"
+	if p.String() != p.clone().String() {
+		t.Errorf("passthrough cloning failed: expected %q, saw %q", p.String(), p.clone().String())
 	}
 }
 
@@ -2161,7 +2391,7 @@ TAG o123-o123
 		},
 
 		// back-ref everywhere
-		{safetyOff, isValid, "(1..$)", "/o(123)/r\\1/g", "unsafe-bref-every",
+		{safetyOff, isValid, "(1..$)", "/o(123)/r${1}/g", "unsafe-bref-every",
 			[]string{
 				"BLB r123-r123\n",
 				"CMT r123-r123\n", "aut@r123-r123.com", "com@r123-r123.com",
@@ -2170,7 +2400,7 @@ TAG o123-o123
 		},
 
 		// back-ref, default comment
-		{safetyOn, isValid, "(1..$ & =C)", "/o(123)/r\\1/cg", "bref-dft-comment",
+		{safetyOn, isValid, "(1..$ & =C)", "/o(123)/r${1}/cg", "bref-dft-comment",
 			[]string{
 				"BLB o123-o123\n",
 				"CMT r123-r123\n", "aut@o123-o123.com", "com@o123-o123.com",
@@ -2179,7 +2409,7 @@ TAG o123-o123
 		},
 
 		// back-ref, commits, comment only, only first
-		{safetyOn, isValid, "(1..$ & =C)", "/o(123)/r\\1/1c", "bref-explicit-first",
+		{safetyOn, isValid, "(1..$ & =C)", "/o(123)/r${1}/1c", "bref-explicit-first",
 			[]string{
 				"BLB o123-o123\n",
 				"CMT r123-o123\n", "aut@o123-o123.com", "com@o123-o123.com",
@@ -2213,7 +2443,9 @@ TAG o123-o123
 			rs.setSelectionSet(test.ss)
 
 			// all tests have valid --regex lines, not checking nil
-			fhook := newFilterCommand(repo, fmt.Sprint("regex ", test.regex))
+			parse := rs.newLineParse(fmt.Sprint("regex ", test.regex),
+				"filter", parseREPO|parseNEEDSELECT|parseNEEDARG, nil)
+			fhook := newFilterCommand(parse)
 
 			rs.chosen().dataTraverse("", rs.selection, fhook.do, fhook.attributes, test.safety, true, control.baton)
 
