@@ -6,7 +6,7 @@ package main
 // evalSelectionSet() that is uses to produce a selection set using
 // the machine.
 //
-// Copyright by Eric S. Raymond
+// SPDX-FileCopyrightText: Copyright Eric S. Raymond <esr@thyrsus.com>
 // SPDX-License-Identifier: BSD-2-Clause
 
 import (
@@ -24,6 +24,12 @@ import (
 	orderedset "github.com/emirpasic/gods/sets/linkedhashset"
 )
 
+const defaultDESCLIMIT = 50
+
+func init() {
+	setUnclean(defaultDESCLIMIT)
+}
+
 type selectionSet struct{ set *orderedset.Set }
 
 type selectionSetIt struct{ orderedset.Iterator }
@@ -77,6 +83,15 @@ func (s selectionSet) Values() []int {
 	return v
 }
 
+func (s selectionSet) Clone() selectionSet {
+	var clone selectionSet
+	it := s.Iterator()
+	for it.Next() {
+		clone.Add(it.Value())
+	}
+	return clone
+}
+
 func (s selectionSet) Contains(x int) bool {
 	return s.set != nil && s.set.Contains(x)
 }
@@ -219,11 +234,13 @@ func (s skipIt) commit() *Commit {
 // unclean tells us how to detect ungitified comments.  Check for Git
 // conventions - require a spacer line after first \n if multiline,
 // subject line no more than 50 chars, body lines no more than 72.
-var unclean = regexp.MustCompile("^[^\n]*\n[^\n]|^.{51,}\n|\n.{73,}")
+var unclean *regexp.Regexp
+
+func setUnclean(desclimit int) {
+	unclean = regexp.MustCompile(fmt.Sprintf("^[^\n]*\n[^\n]|^.{%d,}\n|\n.{73,}", desclimit+1))
+}
 
-//
 // The selection-language parsing code starts here.
-//
 func (rs *Reposurgeon) parseSelectionSet(line string) (machine selEvaluator, rest string) {
 	s := strings.TrimLeft(line, " \t")
 	i := strings.IndexAny(s, " \t")
@@ -336,7 +353,7 @@ func (rs *Reposurgeon) parseTerm() selEvaluator {
 	return term
 }
 
-// Parse a path name to evaluate the set of commits that refer to it.
+// Parse a path name to evaluate the set of commits and blobs that refer to it.
 func (rs *Reposurgeon) parsePathset() selEvaluator {
 	rs.eatWS()
 	if rs.peek() != '[' {
@@ -345,6 +362,7 @@ func (rs *Reposurgeon) parsePathset() selEvaluator {
 	rs.pop()
 	var matcher string
 	depth := 1
+	complement := false
 	for i, c := range rs.line {
 		if c == '[' {
 			depth++
@@ -360,6 +378,10 @@ func (rs *Reposurgeon) parsePathset() selEvaluator {
 	if depth != 0 {
 		panic(throw("command", "malformed path matcher; unbalanced [ and ]"))
 	}
+	if strings.HasPrefix(matcher, "~") {
+		complement = true
+		matcher = matcher[1:]
+	}
 	if strings.HasPrefix(matcher, "/") {
 		end := strings.LastIndexByte(matcher, '/')
 		if end < 1 {
@@ -380,20 +402,20 @@ func (rs *Reposurgeon) parsePathset() selEvaluator {
 			panic(throw("command", "invalid regular expression %s", matcher))
 		}
 		return func(x selEvalState, s selectionSet) selectionSet {
-			return rs.evalPathsetRegex(x, s, search, flags)
+			return rs.evalPathsetRegex(x, s, complement, search, flags)
 		}
 	}
 	return func(x selEvalState, s selectionSet) selectionSet {
-		return rs.evalPathset(x, s, matcher)
+		return rs.evalPathset(x, s, complement, matcher)
 	}
 }
 
 // Resolve a path regex to the set of commits that refer to it.
 func (rs *Reposurgeon) evalPathsetRegex(state selEvalState,
-	preselection selectionSet, search *regexp.Regexp,
+	preselection selectionSet, complement bool, search *regexp.Regexp,
 	flags orderedStringSet) selectionSet {
 	if flags.Contains("c") {
-		return rs.evalPathsetFull(state, preselection,
+		return rs.evalPathsetFull(state, preselection, complement,
 			search, flags.Contains("a"))
 	}
 	all := flags.Contains("a")
@@ -419,7 +441,7 @@ func (rs *Reposurgeon) evalPathsetRegex(state selEvalState,
 					}
 				}
 			}
-			if matches > 0 && (!all || matches == len(paths)) {
+			if (matches > 0 && (!all || matches == len(paths))) != complement {
 				hits.Add(it.Value())
 			}
 		}
@@ -429,7 +451,7 @@ func (rs *Reposurgeon) evalPathsetRegex(state selEvalState,
 
 // Resolve a path name to the set of commits that refer to it.
 func (rs *Reposurgeon) evalPathset(state selEvalState,
-	preselection selectionSet, matcher string) selectionSet {
+	preselection selectionSet, complement bool, matcher string) selectionSet {
 	type vendPaths interface {
 		paths(orderedStringSet) orderedStringSet
 	}
@@ -438,7 +460,7 @@ func (rs *Reposurgeon) evalPathset(state selEvalState,
 	it := preselection.Iterator()
 	for it.Next() {
 		if e, ok := events[it.Value()].(vendPaths); ok &&
-			e.paths(nil).Contains(matcher) {
+			e.paths(nil).Contains(matcher) != complement {
 			hits.Add(it.Value())
 		}
 	}
@@ -446,7 +468,7 @@ func (rs *Reposurgeon) evalPathset(state selEvalState,
 }
 
 func (rs *Reposurgeon) evalPathsetFull(state selEvalState,
-	preselection selectionSet, matchCond *regexp.Regexp,
+	preselection selectionSet, complement bool, matchCond *regexp.Regexp,
 	matchAll bool) selectionSet {
 	// Try to match a regex in the trees. For each commit we remember
 	// only the part of the tree that matches the regex. In most cases
@@ -493,7 +515,7 @@ func (rs *Reposurgeon) evalPathsetFull(state selEvalState,
 			}
 		}
 		matchTrees[c.mark] = tree
-		if tree.isEmpty() == matchAll {
+		if (tree.isEmpty() == matchAll) != complement {
 			result.Add(i)
 		}
 	}
@@ -503,7 +525,6 @@ func (rs *Reposurgeon) evalPathsetFull(state selEvalState,
 // Does an event contain something that looks like a legacy reference?
 func (rs *Reposurgeon) hasReference(event Event) bool {
 	repo := rs.chosen()
-	repo.dollarOnce.Do(func() { repo.parseDollarCookies() })
 	var text string
 	type commentGetter interface{ getComment() string }
 	if g, ok := event.(commentGetter); ok {
@@ -529,11 +550,27 @@ func (rs *Reposurgeon) visibilityTypeletters() map[rune]func(int) bool {
 	e := func(i int) Event {
 		return rs.chosen().events[i]
 	}
-	// Available: AEGKSVWXY
+	isBranchroot := func(c *Commit) bool {
+		if !c.hasParents() {
+			return true
+		} else if c.parentCount() > 1 {
+			return false // Don't count merge commits as roots
+		} else if parent, ok := c.parents()[0].(*Commit); !ok {
+			return false // nor callouts
+		} else {
+			return parent.Branch != c.Branch
+		}
+	}
+	isBisector := func(c *Commit) bool {
+		all := c.repo.commits(undefinedSelectionSet)
+		return c == all[0] || c == all[len(all)-1] || c == all[len(all)/2]
+	}
+	// Available: AGKSVWY
 	return map[rune]func(int) bool{
 		'B': func(i int) bool { _, ok := e(i).(*Blob); return ok },
 		'C': func(i int) bool { _, ok := e(i).(*Commit); return ok },
 		'D': func(i int) bool { p, ok := e(i).(alldel); return ok && p.alldeletes() },
+		'E': func(i int) bool { p, ok := e(i).(*Commit); return ok && isBranchroot(p) },
 		'F': func(i int) bool { c, ok := e(i).(*Commit); return ok && c.childCount() > 1 },
 		'H': func(i int) bool { c, ok := e(i).(*Commit); return ok && !c.hasChildren() },
 		'I': func(i int) bool { p, ok := e(i).(decodable); return ok && !p.decodable() },
@@ -547,6 +584,7 @@ func (rs *Reposurgeon) visibilityTypeletters() map[rune]func(int) bool {
 		'R': func(i int) bool { _, ok := e(i).(*Reset); return ok },
 		'T': func(i int) bool { _, ok := e(i).(*Tag); return ok },
 		'U': func(i int) bool { c, ok := e(i).(*Commit); return ok && c.hasCallouts() },
+		'X': func(i int) bool { c, ok := e(i).(*Commit); return ok && isBisector(c) },
 		'Z': func(i int) bool { c, ok := e(i).(*Commit); return ok && len(c.operations()) == 0 },
 	}
 }
@@ -1435,7 +1473,7 @@ type attrEditAttr interface {
 	name() string
 	email() string
 	date() Date
-	assign(name, email string, date Date)
+	assign(name, email string, date Date) bool
 	remove(Event)
 	insert(after bool, e Event, a Attribution)
 }
@@ -1452,7 +1490,8 @@ func (p *attrEditMixin) name() string  { return p.a.fullname }
 func (p *attrEditMixin) email() string { return p.a.email }
 func (p *attrEditMixin) date() Date    { return p.a.date }
 
-func (p *attrEditMixin) assign(name, email string, date Date) {
+func (p *attrEditMixin) assign(name, email string, date Date) bool {
+	changed := p.a.fullname != name || p.a.email != email || p.a.date != date
 	if len(name) != 0 {
 		p.a.fullname = name
 	}
@@ -1462,6 +1501,7 @@ func (p *attrEditMixin) assign(name, email string, date Date) {
 	if !date.isZero() {
 		p.a.date = date
 	}
+	return changed
 }
 
 func (p *attrEditMixin) minOne(desc string) {
@@ -1503,6 +1543,7 @@ func (p *attrEditAuthor) desc() string { return "author" }
 
 func (p *attrEditAuthor) remove(e Event) {
 	c := e.(*Commit)
+	c.addColor(colorQSET)
 	v := c.authors
 	copy(v[p.pos:], v[p.pos+1:])
 	v[len(v)-1] = Attribution{}
@@ -1511,6 +1552,7 @@ func (p *attrEditAuthor) remove(e Event) {
 
 func (p *attrEditAuthor) insert(after bool, e Event, a Attribution) {
 	c := e.(*Commit)
+	c.addColor(colorQSET)
 	newpos := p.pos
 	if after {
 		newpos++
@@ -1620,7 +1662,7 @@ func (p *AttributionEditor) attributions(e Event) []attrEditAttr {
 		}
 		return v
 	case *Tag:
-		return []attrEditAttr{newAttrEditTagger(x.tagger)}
+		return []attrEditAttr{newAttrEditTagger(&x.tagger)}
 	default:
 		panic(fmt.Sprintf("unexpected event type: %T", e))
 	}
@@ -1761,7 +1803,9 @@ func (p *AttributionEditor) doAssign(eventNo int, e Event, attrs []attrEditAttr,
 		panic(throw("command", "no attribution selected"))
 	}
 	for it := sel.Iterator(); it.Next(); {
-		attrs[it.Value()].assign(name, email, date)
+		if attrs[it.Value()].assign(name, email, date) {
+			e.addColor(colorQSET)
+		}
 	}
 }
 
