@@ -6,6 +6,8 @@
 # Expect the resulting dump to have an add with copyfrom at
 # the last commit, as opposed to a replace.  Verify that
 # the file copy operation leaves the executable bit set.
+#
+# This is not a generator.
 
 # shellcheck disable=SC1091
 . ./common-setup.sh
@@ -24,10 +26,8 @@ done
 # shellcheck disable=SC2004
 shift $(($OPTIND - 1))
 {
-    set -e
-    trap 'svnwrap' EXIT HUP INT QUIT TERM
-    svninit
-    tapcd trunk
+    vc init svn
+    vc stdlayout
     svn mkdir targetdir
     svn mkdir sourcedir
     echo "Source file" >sourcedir/sourcefile.txt
@@ -36,15 +36,15 @@ shift $(($OPTIND - 1))
     svn ci -m "Initial commit of example files"
     svn cp sourcedir/sourcefile.txt targetdir
     svn ci -m "Copy of sourcedir/sourcefile.txt to targetdir."
-    tapcd ../..
-    ls test-checkout$$
+    if [ -x targetdir/sourcefile.txt ]; then executable=yes; fi
+    vc wrap
 } >"/dev/${verbose}" 2>&1
 
 # shellcheck disable=2010
 if [ "$dump" = yes ]
 then
-    svnadmin dump -q test-repo$$
-elif ls -l test-checkout$$/trunk/targetdir/sourcefile.txt | grep x >/dev/null
+    vc export "exec propagation test"
+elif [ "${executable}" = yes ]
 then
     echo "ok - $0: executable permission is as expected"
     exit 0
