@@ -1,6 +1,8 @@
 #!/bin/sh
 ## Test propagation of executable bit by directory copy, second variant
-# This was made from gen-dump2.h. attached to issue #103.
+# Originally made from gen-dump2.sh, attached to issue #103.
+#
+# This is not a generator.
 
 # shellcheck disable=SC1091
 . ./common-setup.sh
@@ -10,20 +12,15 @@ verbose=null
 while getopts dv opt
 do
     case $opt in
-	d) dump=yes;; v) verbose=stdout;;
+	d) dump=yes;;
+	v) verbose=stdout;;
 	*) echo "not ok - $0: unknown flag $opt"; exit 1;;
     esac
 done
 # shellcheck disable=SC2004
 shift $(($OPTIND - 1))
 {
-    trap 'svnwrap' EXIT HUP INT QUIT TERM
-    svinit
-    set -e
-    dir=$(pwd)
-    svnadmin create "${dir}/test-repo$$"
-    svn co "file://${dir}/test-repo$$" test-checkout$$
-    tapcd test-checkout$$
+    vc init svn
     mkdir trunk
     svn add trunk
     svn commit -m "Create trunk."
@@ -37,14 +34,14 @@ shift $(($OPTIND - 1))
     svn up
     svn cp dir1 dir2
     svn commit -m "Copy dir1 to dir2."
-    tapcd ../..
-    echo "I'm here: ${PWD}"
+    if [ -x trunk/dir2/file ]; then executable=yes; fi
+    vc wrap
 } >/dev/$verbose 2>&1
 # shellcheck disable=SC2010
 if [ "$dump" = yes ]
 then
-    svnadmin dump -q test-repo$$
-elif ls -l test-checkout$$/trunk/dir2/file | grep x >/dev/null
+    vc export "exec propagation test"
+elif [ "${executable}" = yes ]
 then
     echo "ok - $0: executable permission is as expected"
     exit 0
