@@ -23,7 +23,7 @@ mkdir "${TMPDIR}/cvs-scratch$$"
 tapcd "${TMPDIR}/cvs-scratch$$"
 
 # Make the workflow file.
-repotool initialize -q hack1 cvs git
+repotool initmake -q hack1 cvs git
 
 # Convert the repository
 # These variables are unset so the following make invocation won't try to
@@ -37,9 +37,7 @@ repotool compare-all hack1-mirror hack1-git >"${TMPDIR}/diff$$"
 if [ -s "${TMPDIR}/diff$$" ]
 then
     echo "not ok - $0: repositories do not compare equal."
-    echo "  --- |"
-    sed <"/tmp/diff$$" -e 's/^/  /'
-    echo "  ..."
+    tapdump "/tmp/diff$$"
     exit 1
 else
     echo "ok - $0: repositories compare equal"
