@@ -19,7 +19,7 @@ tapcd "${TMPDIR}/scratch$$"
 "${testdir}/svn-to-svn" -q -n vanilla-prime <"${testdir}/vanilla.svn"
 
 # Make the workflow file.
-repotool initialize -q vanilla-secundus svn git || ( echo "not ok - $0: initialization failed"; exit 1)
+repotool initmake -q vanilla-secundus svn git || ( echo "not ok - $0: initialization failed"; exit 1)
 
 # Mirror vanilla-prime into vanilla-secundus and invoke standard workflow
 # These variables are unset so the following make invocation won't try to
@@ -33,10 +33,7 @@ repotool compare-all vanilla-secundus-mirror vanilla-secundus-git >"${TMPDIR}/di
 if [ -s "${TMPDIR}/diff$$" ]
 then
     echo "not ok - $0: repositories do not compare equal."
-    echo "  --- |"
-    sed <"/tmp/warnings$$" -e 's/^/  /'
-    sed <"/tmp/diff$$" -e 's/^/  /'
-    echo "  ..."
+    tapdump "/tmp/warnings$$" "/tmp/diff$$"
     exit 1
 else
     echo "ok - $0: repositories compared equal";
