@@ -1,7 +1,7 @@
 // repotool queries and manipulate multiple repository types in a uniform way.
 package main
 
-// Copyright by Eric S. Raymond
+// SPDX-FileCopyrightText: Copyright Eric S. Raymond <esr@thyrsus.com>
 // SPDX-License-Identifier: BSD-2-Clause
 
 import (
@@ -105,7 +105,7 @@ CVS_MODULE = {{.Project}}
 READ_OPTIONS =
 #CHECKOUT_OPTIONS = --ignore-externals
 DUMPFILTER = cat
-VERBOSITY = "set progress"
+VERBOSITY = "set flag progress"
 REPOSURGEON = reposurgeon
 LOGFILE = conversion.log
 
@@ -121,7 +121,7 @@ default: {{.Project}}-{{.TargetVCS}}
 
 # Build the repository from the stream dump
 {{.Project}}-{{.TargetVCS}}: {{.Project}}.{{.SourceVCS}} {{.Project}}.opts {{.Project}}.lift {{.Project}}.map $(EXTRAS)
-	$(REPOSURGEON) $(VERBOSITY) 'logfile $(LOGFILE)' 'script {{.Project}}.opts' "read $(READ_OPTIONS) <{{.Project}}.{{.SourceVCS}}" 'authors read <{{.Project}}.map' 'sourcetype {{.SourceVCS}}' 'prefer git' 'script {{.Project}}.lift' 'legacy write >{{.Project}}.fo' 'rebuild {{.Project}}-{{.TargetVCS}}'
+	$(REPOSURGEON) $(VERBOSITY) 'set logfile $(LOGFILE)' 'do {{.Project}}.opts' "read $(READ_OPTIONS) <{{.Project}}.{{.SourceVCS}}" 'authors read <{{.Project}}.map' 'select {{.SourceVCS}}' 'prefer git' 'do {{.Project}}.lift' 'legacy write >{{.Project}}.fo' 'rebuild {{.Project}}-{{.TargetVCS}}'
 
 # Build a stream dump from the local mirror
 {{.Project}}.{{.SourceVCS}}: {{.Project}}-mirror
@@ -176,7 +176,10 @@ gitk: {{.Project}}-git
 # This repack call is the active part of gc --aggressive.  This call is
 # tuned for very large repositories.
 gc: {{.Project}}-git
-	cd {{.Project}}-git; time git -c pack.threads=1 repack -AdF --window=1250 --depth=250
+	@cd {{.Project}}-git;
+	@echo "Before: $$(du -s -h .)"
+	@time git -c pack.threads=1 repack -AdF --window=1250 --depth=250
+	@echo "After: $$(du -s -h .)"
 `
 
 var acceptMissing bool
@@ -191,6 +194,7 @@ var progress bool
 
 var branch string
 var comparemode string
+var message string
 var refexclude string
 var revision string
 var basedir string
@@ -324,7 +328,7 @@ func makeStub(name string, contents string) {
 	fp.WriteString(contents)
 }
 
-func initialize(args []string) {
+func initmake(args []string) {
 	WriteSupport := newStringSet()
 	for _, vcs := range vcstypes {
 		if vcs.importer != "" {
@@ -340,11 +344,11 @@ func initialize(args []string) {
 	// Hacky special case implemented through extractor class
 	ReadSupport.Add("hg")
 	if verbose {
-		fmt.Printf("initialize args: %v\n", args)
+		fmt.Printf("initmake args: %v\n", args)
 	}
 	var squishy squishyParts
 	if len(args) < 1 {
-		croak("initialize requires a project name.")
+		croak("initmake requires a project name.")
 	}
 	project, args := args[0], args[1:]
 	squishy.Project = project
@@ -528,13 +532,6 @@ func mirror(args []string) {
 		// the clone operation will create a live Git repository capturing the
 		// Perforce history.
 		operand = operand[3:]
-		if mirrordir == "" {
-			locald = filepath.Join(pwd, filepath.Base(operand)+"-mirror")
-		} else if mirrordir[0] == os.PathSeparator {
-			locald = mirrordir
-		} else {
-			locald = filepath.Join(pwd, mirrordir)
-		}
 		cmd := ""
 		legend := ""
 		if isdir(mirrordir) {
@@ -1193,10 +1190,10 @@ var helpdict = map[string]struct {
 	oneliner string
 	text     string
 }{
-	"initialize": {
-		"initialize",
+	"initmake": {
+		"initmake",
 		"create Makefile and stub files for standard conversion workflow",
-		`The "initialize" option takes a project name (and, optionally,
+		`The "initmake" option takes a project name (and, optionally,
 following source and target VCS types) and generates a
 Makefile that will sequence various steps of a repository
 conversion. It also generates stub lift and options files. This is
@@ -1248,8 +1245,14 @@ slashes after the "file:" prefix!
 	"branches": {
 		"branches",
 		"list repository branch names",
-		`The 'branches' option, run from within a repository directory ,
+		`The 'branches' action, run from within a repository directory ,
 returns a list of the repository's branch names.
+`},
+	"tags": {
+		"branches",
+		"list repository tag names",
+		`The 'tags' action, run from within a repository directory ,
+returns a list of the repository's tag names.
 `},
 	"checkout": {
 		"checkout [-r rev] [-t tag] [-b branch] [-o option]",
@@ -1320,7 +1323,7 @@ With a following argument that is a command name, display detailed help for that
 }
 
 var narrativeOrder []string = []string{
-	"initialize",
+	"initmake",
 	"export",
 	"mirror",
 	"branches",
@@ -1329,6 +1332,7 @@ var narrativeOrder []string = []string{
 	"compare-tags",
 	"compare-branches",
 	"compare-all",
+	"tags",
 	"version",
 	"help",
 }
@@ -1362,6 +1366,7 @@ func main() {
 	flags.StringVar(&branch, "b", "", "select branch for checkout or comparison")
 	flags.StringVar(&basedir, "d", "", "chdir to the argument repository path before doing checkout")
 	flags.StringVar(&refexclude, "e", "", "exclude pattern for tag and branch names.")
+	flags.StringVar(&message, "m", "", "argument is commit message text")
 	flags.StringVar(&revision, "r", "", "select revision for checkout or comparison")
 	flags.StringVar(&tag, "t", "", "select tag for checkout or comparison")
 	flags.StringVar(&passthrough, "o", "", "option passthrough")
@@ -1395,8 +1400,8 @@ func main() {
 	}
 
 	args := flags.Args()
-	if operation == "initialize" {
-		initialize(args)
+	if operation == "initmake" {
+		initmake(args)
 	} else if operation == "export" {
 		export()
 	} else if operation == "mirror" {
